import matplotlib.pyplot as plt
import seaborn as sns



class PlotUtilities():
    
    def plot_stellar_spectrum(self):
        raise NotImplementedError('This method is not implemented yet.')
        ...
        
    def plot_planetary_spectrum(self):
        raise NotImplementedError('This method is not implemented yet.')
    
    def plot_transmission_spectrum(self):
        raise NotImplementedError('This method is not implemented yet.')
    
    def plot_RM_effect(self):
        raise NotImplementedError('This method is not implemented yet.')
    ...