from datetime import datetime, timedelta
import astropy.time as astime

def convert_to_ESO_absolute_interval(time_start, time_end):
    raise
    between(2025-12-25T04:03, 2025-12-25T08:39, 1,"TOI-470 b")
    return f"{time_start.strftime('%Y-%m-%dT%H:%M')}/{time_end.strftime('%Y-%m-%dT%H:%M')}"
    ...